//
// Condition.cpp
//
// $Id: //poco/1.3/Foundation/src/Condition.cpp#1 $
//
// Library: Foundation
// Package: Threading
// Module:  Condition
//
// Copyright (c) 2007, Applied Informatics Software Engineering GmbH.
// and Contributors.
//
// Permission is hereby granted, free of charge, to any person or organization
// obtaining a copy of the software and accompanying documentation covered by
// this license (the "Software") to use, reproduce, display, distribute,
// execute, and transmit the Software, and to prepare derivative works of the
// Software, and to permit third-parties to whom the Software is furnished to
// do so, all subject to the following:
// 
// The copyright notices in the Software and this entire statement, including
// the above license grant, this restriction and the following disclaimer,
// must be included in all copies of the Software, in whole or in part, and
// all derivative works of the Software, unless such copies or derivative
// works are solely in the form of machine-executable object code generated by
// a source language processor.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
// FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
// DEALINGS IN THE SOFTWARE.
//


#include "Poco/Condition.h"


namespace Poco {


Condition::Condition()
{
}

Condition::~Condition()
{
}


void Condition::signal()
{
	FastMutex::ScopedLock lock(_mutex);
	
	if (!_waitQueue.empty())
	{
		_waitQueue.front()->set();
		dequeue();
	}
}


void Condition::broadcast()
{
	FastMutex::ScopedLock lock(_mutex);
	
	for (WaitQueue::iterator it = _waitQueue.begin(); it != _waitQueue.end(); ++it)
	{
		(*it)->set();
	}
	_waitQueue.clear();
}


void Condition::enqueue(Event& event)
{
	_waitQueue.push_back(&event);
}


void Condition::dequeue()
{
	_waitQueue.pop_front();
}


void Condition::dequeue(Event& event)
{
	for (WaitQueue::iterator it = _waitQueue.begin(); it != _waitQueue.end(); ++it)
	{
		if (*it == &event)
		{
			_waitQueue.erase(it);
			break;
		}
	}
}


} // namespace Poco
